
/* socket client welke data ontvangt van een socket server */
/* (c)2005 E.J Boks */
/* Hogeschool van Arnhem en Nijmegen  */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <netdb.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>

#include "algdef.h"

#define InternetPoort 3490 // the port client will be connecting to
#define MaxDataGrootte 500 // max number of bytes we can get at once


/***************************** Niet-API funkties ***************************/

static FoutCode clientVerbindMetServer(char *hostNaam,
                                int *sockfd)
{
    struct hostent *he;
    struct sockaddr_in serverAdres; // connector\x{2019}s address information


    if (Null == (he=gethostbyname(hostNaam)) ) 
    {
        herror("gethostbyname");
        return(Fout);
    }

    if (-1 == (*sockfd = socket(PF_INET, SOCK_STREAM, 0)))
    {
        perror("socket");
        return(Fout);
    }

    serverAdres.sin_family = AF_INET; 
    serverAdres.sin_port = htons(InternetPoort); 
    serverAdres.sin_addr = *((struct in_addr *)he->h_addr);

    //memset(&(serverAdres.sin_zero), \x{2019}\0\x{2019}, 8); /* struct voor de rest gelijk steller aan 0 */

    if (-1 == connect(*sockfd, 
                      (struct sockaddr *)&serverAdres,
                      sizeof(struct sockaddr))) 
    {
        perror("connect");
        return(Fout);
    }
    else return(Ok);
}


static FoutCode clientSluitVerbinding(int sockfd)
{

    close(sockfd);

    return(Ok);
}


static FoutCode clientOntvangData(int sockfd,
                           UInt8 *buf,
                           UInt16 maxBufGrootte)
{
    Int16 numbytes;
    UInt16 lengte;

    /* ontvang eerst de lengte */
    if (-1 == (numbytes=recv(sockfd, &lengte, sizeof(UInt16), 0)) ) 
    {
        perror("clientOntvangData ontvang lengte");
        return(Fout);
    }

    if (lengte > MaxDataGrootte)
    {
        fprintf(stderr,"(clientOntvangData) Pakket groter dan de gealloceerde buffer.\n");
        return(Fout);
    }
    else if (-1 == (numbytes=recv(sockfd, buf, lengte, 0)) ) /* ontvang vervolgens de data */
    {
        perror("clientOntvangData ontvang data");
        return(Fout);
    }

    return(Ok);

}


/************************** API ****************************/


/* Voorbeeldprogramma */
static unsigned char ontvangstBuf[MaxDataGrootte];

int main(int argc, char *argv[])
{
    int sockfd;

    if (argc != 2) 
    {
        fprintf(stderr,"Gebruik: %s <servername>\n", argv[0]);
        return(1);
    }
    else
    {

        /* Maak verbinding */
        if (Ok != clientVerbindMetServer(argv[1],
                                         &sockfd ))
            return(2);

        /* ontvang testboodschap */
        if (Ok != clientOntvangData(sockfd,
                                    ontvangstBuf,
                                    MaxDataGrootte))
            return(3);
        else
        {
            printf("De ontvangen data is :%s\n",ontvangstBuf);
            if (Ok != clientSluitVerbinding(sockfd))
                return(4);
            else return(0);
        }


    }
}

