
/************************************************************************/
/* Algemene definities voor operatie                                    */
/* Alle software in dit bestand is                                      */
/* (c) 1998-2001 Oresis Communications                                  */
/* (c) 2001-2005 Kiwanda Embedded Systemen                              */
/* $Id: algdef.h,v 1.11 2005/08/11 06:46:39 oldenburgh Exp $            */
/************************************************************************/

#ifndef ALGDEF_H
#define ALGDEF_H


#include <stdio.h>
//#include <assert.h>

/* standaard definities */

#ifndef PI
#define PI  3.141592654
#endif

#ifdef  FALSE
#define False		FALSE
#else
#define False		0
#endif

#ifdef  TRUE
#define True		TRUE
#else
#define True		1
#endif

#ifndef NULL
#define NULL            0
#endif

#ifndef Null
#define Null            NULL
#endif

#define forever         while(1==1)

/* Basic Types
 */
typedef char	 		Char;
typedef unsigned char		UChar;
typedef unsigned int		UInt;
typedef int			Int;
typedef float			Float;
typedef double			Double;

/* Sized Types
 */
typedef signed char		Int8;
typedef unsigned char		UInt8;
typedef signed short		Int16;
typedef unsigned short		UInt16;
typedef signed long 		Int32;
typedef unsigned long		UInt32;
typedef long long		Int64;  /* wellicht niet nodig voor een 8 bit processor !*/
typedef unsigned long long	UInt64; 

/* Bool Type  */
typedef UInt32			Bool32;	
typedef UInt16			Bool16;
typedef UInt8			Bool8;


/* algemene funktie foutmelding */

typedef enum
{
    Ok=0,
    Waarschuwing,
    Fout,
    Ernstig,
    Rampzalig,
    Reserve
} FoutCode;

/* bitsgewijze operatie macros . Geef bits aan als nummer en niet als 2 tot de macht (nummer) */ 
#define SetBit(x,y)          (x |= (1<<y)) /* Set bit y in byte x*/ 
#define ClearBit(x,y)        (x &= (~(1<<(y)) )) /* Clear bit y in byte x*/ 
#define CheckBit(x,y)        (x & (1<<y))  /* Check bit y in byte x*/
#define EnterBit(x,y,b)      ( (b) ? SetBit(x,y) : ClearBit(x,y) ) /* zet bit y in byte x gelijk aan b */
#define CopyBit(a,b,x,y)     EnterBit((x),(y), (CheckBit(a,b)) ) /* kopieer bit b in byte a naar bit y in byte x */  //    (x =  (x&(~(1<<(y))))  | ((a & (1<<b) ) << (y-b) ) )  /* kopieer bit b in byte a naar bit y in byte x */
#define ToggleBit(x,y)       (x = (x ^ (1<<y)));  /* verander bit y in byte x */


/* Serieele hulpjes */
#define HoogWoord(woord)    ((UInt16) 0xffff & (woord >> 16))  /* In een 32 bit geheugen, is dit het Most Significant Word (16 bit) */
#define LaagWoord(woord)    ((UInt16) 0xffff & woord)          /* In een 32 bit geheugen, is dit het Least Significant Word (16 bit) */
#define HoogByte(woord)    ((UInt8) 0xff & (woord >> 8))       /* In een 16 bit geheugen, is dit het Most Significant Byte (8 bit) */
#define LaagByte(woord)    ((UInt8) 0xff & woord)              /* In een 16 bit geheugen, is dit het Least Significant Byte (8 bit) */

#define Hoog              1
#define Laag              0
                              
#define ReturnIsOk(a)  ( ( (a) == Ok ) ? True : False )
#define ReturnIsNietOk(a)  ( ( (a) != Ok ) ? True : False )


#endif  /* ALGDEF_H */





