/* socket server voor databestand aan studenten PCs */
/* (c)2005 E.J Boks */
/* Enhanced by Erik de Mare: */
/*      - added IPv6 support (08-03-2006) */
/* Hogeschool van Arnhem en Nijmegen  */


#include <stdlib.h>
#include <stdint.h>
#include <pthread.h>
#include <string.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/inet.h>


#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <stdio.h>
#include <errno.h>
#include <time.h>
#include <unistd.h>

const char  *LuisterPoort =  "3490"; /*  the port users will be connecting to */
#define BackLog 10          /* how many pending connections queue will hold */
#define Null            NULL


typedef struct
{
    uint8_t id;              /* Draad ID */

    int sockfd;            /* socket id */
    char *data;            /* Plek waar de data zich bevindt */
    uint16_t dataGrootte;    /* lengte van de data */
} ThreadArgument;

static pthread_mutex_t dataBestandMutex = PTHREAD_MUTEX_INITIALIZER;

static void dataVoeder(ThreadArgument *arg)
{
    unsigned int verzondenLengte;

    printf("Thread %d neemt deze taak op zich.\n",arg->id);

    if (0 != pthread_mutex_lock( &dataBestandMutex))
    {
        perror("(dataVoeder) kon de mutex niet sluiten.\n");
        exit(1);
    }
    else
    {

        /* verstuur eerst de lengte van het databestand */
        verzondenLengte = send(arg->sockfd,
                               &(arg->dataGrootte),
                               sizeof(uint16_t),
                               0);

        /* verstuur dan het databestand naar de client */
        verzondenLengte = send(arg->sockfd,
                               arg->data,
                               arg->dataGrootte,
                               0);

        if (0 != pthread_mutex_unlock( &dataBestandMutex))
        {
            perror("(dataVoeder) kon de mutex niet ontsluiten.\n");
            exit(2);
        }

    }

}


int main(int argc, char *argv[] )
{

    /* databestand */
    FILE *dataBestand;
    struct stat fileStats;

    /* Deze machine */
    struct addrinfo hints, *res;
    char clienthost[NI_MAXHOST];
    char clientservice[NI_MAXSERV];
    /* Socket */
    int sockFd, newFd, error; /* listen on sock_fd, new connection on new_fd */
    struct sockaddr_storage clientaddr;
    socklen_t addrlen;

    /* Thread */
    uint8_t threadId=0;             /* thread teller */
    ThreadArgument argument;
    pthread_t draad; 
    memset(&hints, 0, sizeof(struct addrinfo));
        hints.ai_flags    = AI_PASSIVE;
        hints.ai_family   = AF_UNSPEC;
        hints.ai_socktype = SOCK_STREAM;

    switch(argc)
    {
        case 2:
                error = getaddrinfo( NULL, LuisterPoort, &hints, &res);
                if (error != 0) {
                        fprintf(stderr,
                                "getaddrinfo error:: [%s]\n",
                                gai_strerror(error));
                        return -1;
                }
                break;

        case 3:
            error = getaddrinfo((const char *)argv[2], LuisterPoort, &hints, &res);
                        if (error != 0) {
                                fprintf(stderr,
                                        "getaddrinfo error:: [%s]\n",
                                        gai_strerror(error));
                                return -1;
                        }
            break;

        default:
            fprintf(stderr,"Gebruik: %s <bestandnaam> [<ip-adres>] \n", argv[0]);
            return(1);
            break;

    }

    /* Het databestand voor de client */
    if (Null == (dataBestand = fopen(argv[1],"r")))
    {
        fprintf(stderr,"(main) kon bestand %s niet openen",argv[1]);
        return(5);
    }
    else
    {

        /* lees de bestand attributen */
        if(-1 == stat(argv[1],&fileStats))
        {
            perror("(main) kon bestand attributen niet lezen");
            return(5);
        }
        else
        {
            argument.dataGrootte = fileStats.st_size;
            if (Null == (argument.data = (char *)malloc(argument.dataGrootte*sizeof(char))))
            {
                perror("(main) kon geen geheugen krijgen");
                return(5);
            }
            else if (Null == fgets(argument.data, argument.dataGrootte , dataBestand))
            {
                perror("(main) kon databestand niet inlezen");
                return(5);
            }
        }               
       
    }


    sockFd=-1;
    while (res) {
        sockFd = socket(res->ai_family,
                        res->ai_socktype,
                        res->ai_protocol);

        if (sockFd >= 0) {
            if (bind(sockFd, res->ai_addr, res->ai_addrlen) == 0)
                break;

            close(sockFd);
            sockFd=-1;
        }
        res = res->ai_next;
    }
    if (sockFd < 0) {
        fprintf(stderr,
                "socket error:: could not open socket\n");
        return -1;
    }


    if ( -1 == listen(sockFd, BackLog))
    {
        perror("(main) Kon niet luisteren naar de socket");
        return(3);
    }

    printf("Wij luisteren naar verzoeken op poort %s.\n",
           LuisterPoort );


    while (1)    /* eindeloze lus */
    {
                addrlen = sizeof(clientaddr);
        if (-1 == (newFd = accept(sockFd, (struct sockaddr *)&clientaddr, &addrlen)))
        {
            perror("(main) fout bij het accepteren van een verbinding");
            continue;
        }
        else
        {
                memset(clienthost, 0, sizeof(clienthost));
                memset(clientservice, 0, sizeof(clientservice));
                getnameinfo((struct sockaddr *)&clientaddr, addrlen, clienthost, sizeof(clienthost), clientservice, sizeof(clientservice), NI_NUMERICHOST);
            printf("Socket server: verbinding gemaakt met %s poort %s\n",
                   clienthost,clientservice);


            argument.sockfd = newFd;
            argument.id = threadId++;

            /* maak een draad met de nieuwe descriptor */
            if ( 0 != pthread_create(&draad,
                                     (const pthread_attr_t *) Null,
                                     (void *) &dataVoeder,
                                     (void *) &argument) )
            {
                perror("(main) kon geen nieuwe thread aanmaken");
                return(4);
            }
           
    
        }


    }

 
    return(0);

}



